// src/index.ts
import {
  delimitedIndent,
  foldInside,
  foldNodeProp,
  indentNodeProp,
  LanguageSupport,
  LRLanguage
} from "@codemirror/language";
import { styleTags, tags as t } from "@lezer/highlight";

// esbuild-plugin-lezer:/Users/nvie/Projects/liveblocks/liveblocks-schema/packages/codemirror-language/src/syntax.grammar
import { LRParser } from "@lezer/lr";
var spec_name = { __proto__: null, type: 10 };
var parser = LRParser.deserialize({
  version: 14,
  states: "&fOYQPOOO_QQO'#C_QYQPOOOOQO'#C`'#C`OOQO'#Co'#CoOOQO'#Cb'#CbOdQPO,58yOOQO-E6m-E6mOlQPO'#CyOOQO'#Cc'#CcOOQO1G.e1G.eOqQPO1G.eOOQO'#Cf'#CfOvQPO'#CeOOQO'#Cp'#CpO!OQPO,59eOOQO7+$P7+$PO!WQQO,59PO!`QPO,59POOQO-E6n-E6nOOQO1G/P1G/POOQO'#Ci'#CiO!eQPO'#ChOOQO'#Ch'#ChO!vQPO1G.kO!WQQO1G.kO!WQQO,59VOOQO'#Cr'#CrO#RQPO,59XOOQO7+$V7+$VO#aQPO7+$VO#lQQO'#C{O#tQQO'#ChO$SQPO1G.qOOQO-E6p-E6pOOQO<<Gq<<GqO!WQQO'#CqO$XQQO,59gO$aQQO,59XOOQO7+$]7+$]OOQO,59],59]OOQO-E6o-E6o",
  stateData: "$l~OiOSPOS~OTRO~OkTO~OWWOlZO~Oj[O~OWWO~OZaOnbO~ObdOj[O~OWWOkeO~OZiO~O`jOqkOb[Xj[Xr[X~OrmObXijXi~OqkObaajaaraa~OrsObXqjXq~OptO^oX~O`jOqkO^[Xp[X~O^wO~OptO^oa~OqkO^aapaa~O",
  goto: "#opPPPquPy|P!Y!^P!b!nP!SP!SP!v!|#S#YPPPPPP#dP#lTSOQTPOQRUPQYUQ`ZXgaijtT^W_T]W_QhaQniQojRxtSfaiTpjtQQORVQQ_WRc_QuoRyuQlfSrlvRvp]XUZaijtRqj",
  nodeNames: "\u26A0 Comment Document ObjectTypeDefinition TypeKeyword type NamedTypeDeclaration FieldsDefinition { ValueDefinition Name : Type NamedType > GenericType < ArrayType }",
  maxTerm: 34,
  skippedNodes: [0, 1],
  repeatNodeCount: 4,
  tokenData: "&U~RiX^!ppq!pst#e|}#|!P!Q$T![!]$Z!]!^$`!^!_$e!_!`$j!`!a$o!a!b$t!c!}$y!}#O%^#R#S$y#T#o%i#o#p%z#q#r&P#y#z!p$f$g!p#BY#BZ!p$IS$I_!p$I|$JO!p$JT$JU!p$KV$KW!p&FU&FV!p~!uYi~X^!ppq!p#y#z!p$f$g!p#BY#BZ!p$IS$I_!p$I|$JO!p$JT$JU!p$KV$KW!p&FU&FV!p~#jSP~OY#eZ;'S#e;'S;=`#v<%lO#e~#yP;=`<%l#eR$TOpQrP~$WP!P!Q#e~$`OZ~P$eOrP~$jO`~~$oOl~~$tO^~~$yOn~R%QSkQjP!Q![$y!c!}$y#R#S$y#T#o$y~%aP#P#Q%d~%iOq~P%nSjP!Q![%i!c!}%i#R#S%i#T#o%i~&POW~~&UOb~",
  tokenizers: [0, 1],
  topRules: { "Document": [0, 2] },
  specialized: [{ term: 26, get: (value) => spec_name[value] || -1 }],
  tokenPrec: 0
});

// src/linter.ts
import { linter as lint } from "@codemirror/lint";
import { getDiagnostics } from "@liveblocks/schema";
var linter = lint(
  (view) => {
    const code = view.state.doc.toString();
    const diagnostics = getDiagnostics(code).map((diagnostic) => {
      var _a;
      return {
        from: diagnostic.range[0].offset,
        to: diagnostic.range[1].offset,
        message: diagnostic.message,
        severity: diagnostic.severity,
        actions: (_a = diagnostic.suggestions) == null ? void 0 : _a.map((suggestion) => {
          if (suggestion.type === "replace") {
            return {
              name: "Replace",
              apply: (view2, from, to) => {
                view2.dispatch({
                  changes: { from, to, insert: suggestion.name }
                });
              }
            };
          } else if (suggestion.type === "add-object-type-def") {
            return {
              name: "Add definition",
              apply: (view2) => {
                const insert = `

type ${suggestion.name} {
  
}
`;
                const to = code.length;
                const from = code.trimEnd().length;
                view2.dispatch({
                  changes: {
                    from: to,
                    insert
                  },
                  selection: { anchor: from + insert.length - 3 }
                });
              }
            };
          } else {
            return;
          }
        }).filter(Boolean)
      };
    });
    return diagnostics;
  },
  { delay: 200 }
);

// src/index.ts
var parentheses = "( )";
var braces = "{ }";
var angleBrackets = "< >";
var punctuations = `${parentheses} ${braces} ${angleBrackets} : = ?`;
var blocks = `FieldsDefinition ${braces}`;
var generic = `GenericType ${angleBrackets}`;
var LiveblocksSchema = LRLanguage.define({
  name: "liveblocks-schema",
  parser: parser.configure({
    props: [
      indentNodeProp.add({
        [blocks]: delimitedIndent({ closing: "}", align: true })
      }),
      indentNodeProp.add({
        [generic]: delimitedIndent({ closing: ">", align: false })
      }),
      foldNodeProp.add({
        [blocks]: foldInside
      }),
      styleTags({
        Comment: t.lineComment,
        Name: t.name,
        NamedType: t.typeName,
        GenericType: t.special(t.typeName),
        NamedTypeDeclaration: t.variableName,
        type: t.keyword,
        separator: t.separator,
        comma: t.separator,
        [parentheses]: t.paren,
        [braces]: t.brace,
        [angleBrackets]: t.angleBracket,
        [punctuations]: t.punctuation
      })
    ]
  }),
  languageData: {
    commentTokens: { line: "#" },
    closeBrackets: { brackets: ["{", "<", "[", '"'] },
    indentOnInput: /^\s*(\{|\})$/
  }
});
function liveblocksSchema() {
  return new LanguageSupport(LiveblocksSchema);
}
export {
  LiveblocksSchema,
  linter,
  liveblocksSchema
};
